@@ -24,10 +24,13 @@ import os
 import random
 import shlex
 import signal
+import sys
 import time
 
+import enum
 from oslo_utils import importutils
 from oslo_utils import strutils
+from oslo_utils import timeutils
 import six
 
 from oslo_concurrency._i18n import _
@@ -60,26 +63,33 @@ class UnknownArgumentError(Exception):
 class ProcessExecutionError(Exception):
     def __init__(self, stdout=None, stderr=None, exit_code=None, cmd=None,
                  description=None):
+        super(ProcessExecutionError, self).__init__(
+            stdout, stderr, exit_code, cmd, description)
         self.exit_code = exit_code
         self.stderr = stderr
         self.stdout = stdout
         self.cmd = cmd
         self.description = description
 
+    def __str__(self):
+        description = self.description
         if description is None:
             description = _("Unexpected error while running command.")
+
+        exit_code = self.exit_code
         if exit_code is None:
             exit_code = '-'
+
         message = _('%(description)s\n'
                     'Command: %(cmd)s\n'
                     'Exit code: %(exit_code)s\n'
                     'Stdout: %(stdout)r\n'
                     'Stderr: %(stderr)r') % {'description': description,
-                                             'cmd': cmd,
+                                             'cmd': self.cmd,
                                              'exit_code': exit_code,
-                                             'stdout': stdout,
-                                             'stderr': stderr}
-        super(ProcessExecutionError, self).__init__(message)
+                                             'stdout': self.stdout,
+                                             'stderr': self.stderr}
+        return message
 
 
 class NoRootWrapSpecified(Exception):
@@ -95,8 +105,77 @@ def _subprocess_setup(on_preexec_fn):
         on_preexec_fn()
 
 
-LOG_ALL_ERRORS = 1
-LOG_FINAL_ERROR = 2
+@enum.unique
+class LogErrors(enum.IntEnum):
+    """Enumerations that affect if stdout and stderr are logged on error.
+
+    .. versionadded:: 2.7
+    """
+
+    #: No logging on errors.
+    DEFAULT = 0
+
+    #: Log an error on **each** occurence of an error.
+    ALL = 1
+
+    #: Log an error on the last attempt that errored **only**.
+    FINAL = 2
+
+
+# Retain these aliases for a number of releases...
+LOG_ALL_ERRORS = LogErrors.ALL
+LOG_FINAL_ERROR = LogErrors.FINAL
+LOG_DEFAULT_ERROR = LogErrors.DEFAULT
+
+
+class ProcessLimits(object):
+    """Resource limits on a process.
+
+    Attributes:
+
+    * address_space: Address space limit in bytes
+    * core_file_size: Core file size limit in bytes
+    * cpu_time: CPU time limit in seconds
+    * data_size: Data size limit in bytes
+    * file_size: File size limit in bytes
+    * memory_locked: Locked memory limit in bytes
+    * number_files: Maximum number of open files
+    * number_processes: Maximum number of processes
+    * resident_set_size: Maximum Resident Set Size (RSS) in bytes
+    * stack_size: Stack size limit in bytes
+
+    This object can be used for the *prlimit* parameter of :func:`execute`.
+    """
+
+    _LIMITS = {
+        "address_space": "--as",
+        "core_file_size": "--core",
+        "cpu_time": "--cpu",
+        "data_size": "--data",
+        "file_size": "--fsize",
+        "memory_locked": "--memlock",
+        "number_files": "--nofile",
+        "number_processes": "--nproc",
+        "resident_set_size": "--rss",
+        "stack_size": "--stack",
+    }
+
+    def __init__(self, **kw):
+        for limit in self._LIMITS.keys():
+            setattr(self, limit, kw.pop(limit, None))
+
+        if kw:
+            raise ValueError("invalid limits: %s"
+                             % ', '.join(sorted(kw.keys())))
+
+    def prlimit_args(self):
+        """Create a list of arguments for the prlimit command line."""
+        args = []
+        for limit in self._LIMITS.keys():
+            val = getattr(self, limit)
+            if val is not None:
+                args.append("%s=%s" % (self._LIMITS[limit], val))
+        return args
 
 
 def execute(*cmd, **kwargs):
@@ -136,17 +215,19 @@ def execute(*cmd, **kwargs):
     :param loglevel:        log level for execute commands.
     :type loglevel:         int.  (Should be logging.DEBUG or logging.INFO)
     :param log_errors:      Should stdout and stderr be logged on error?
-                            Possible values are None=default,
-                            LOG_FINAL_ERROR, or LOG_ALL_ERRORS. None
-                            implies no logging on errors. The values
-                            LOG_FINAL_ERROR and LOG_ALL_ERRORS are
-                            relevant when multiple attempts of command
-                            execution are requested using the
-                            'attempts' parameter. If LOG_FINAL_ERROR
-                            is specified then only log an error on the
-                            last attempt, and LOG_ALL_ERRORS requires
-                            logging on each occurence of an error.
-    :type log_errors:       integer.
+                            Possible values are
+                            :py:attr:`~.LogErrors.DEFAULT`,
+                            :py:attr:`~.LogErrors.FINAL`, or
+                            :py:attr:`~.LogErrors.ALL`. Note that the
+                            values :py:attr:`~.LogErrors.FINAL` and
+                            :py:attr:`~.LogErrors.ALL`
+                            are **only** relevant when multiple attempts of
+                            command execution are requested using the
+                            ``attempts`` parameter.
+    :type log_errors:       :py:class:`~.LogErrors`
+    :param binary:          On Python 3, return stdout and stderr as bytes if
+                            binary is True, as Unicode otherwise.
+    :type binary:           boolean
     :param on_execute:      This function will be called upon process creation
                             with the object as a argument.  The Purpose of this
                             is to allow the caller of `processutils.execute` to
@@ -165,11 +246,35 @@ def execute(*cmd, **kwargs):
                             subprocess.Popen on windows (throws a
                             ValueError)
     :type preexec_fn:       function()
+    :param prlimit:         Set resource limits on the child process. See
+                            below for a detailed description.
+    :type prlimit:          :class:`ProcessLimits`
     :returns:               (stdout, stderr) from process execution
     :raises:                :class:`UnknownArgumentError` on
                             receiving unknown arguments
     :raises:                :class:`ProcessExecutionError`
     :raises:                :class:`OSError`
+
+    The *prlimit* parameter can be used to set resource limits on the child
+    process.  If this parameter is used, the child process will be spawned by a
+    wrapper process which will set limits before spawning the command.
+
+    .. versionchanged:: 3.4
+       Added *prlimit* optional parameter.
+
+    .. versionchanged:: 1.5
+       Added *cwd* optional parameter.
+
+    .. versionchanged:: 1.9
+       Added *binary* optional parameter. On Python 3, *stdout* and *stdout*
+       are now returned as Unicode strings by default, or bytes if *binary* is
+       true.
+
+    .. versionchanged:: 2.1
+       Added *on_execute* and *on_completion* optional parameters.
+
+    .. versionchanged:: 2.3
+       Added *preexec_fn* optional parameter.
     """
 
     cwd = kwargs.pop('cwd', None)
@@ -184,9 +289,13 @@ def execute(*cmd, **kwargs):
     shell = kwargs.pop('shell', False)
     loglevel = kwargs.pop('loglevel', logging.DEBUG)
     log_errors = kwargs.pop('log_errors', None)
+    if log_errors is None:
+        log_errors = LogErrors.DEFAULT
+    binary = kwargs.pop('binary', False)
     on_execute = kwargs.pop('on_execute', None)
     on_completion = kwargs.pop('on_completion', None)
     preexec_fn = kwargs.pop('preexec_fn', None)
+    prlimit = kwargs.pop('prlimit', None)
 
     if isinstance(check_exit_code, bool):
         ignore_exit_code = not check_exit_code
@@ -197,7 +306,9 @@ def execute(*cmd, **kwargs):
     if kwargs:
         raise UnknownArgumentError(_('Got unknown keyword args: %r') % kwargs)
 
-    if log_errors not in [None, LOG_ALL_ERRORS, LOG_FINAL_ERROR]:
+    if isinstance(log_errors, six.integer_types):
+        log_errors = LogErrors(log_errors)
+    if not isinstance(log_errors, LogErrors):
         raise InvalidArgumentError(_('Got invalid arg log_errors: %r') %
                                    log_errors)
 
@@ -214,12 +325,27 @@ def execute(*cmd, **kwargs):
             cmd = shlex.split(root_helper) + list(cmd)
 
     cmd = [str(c) for c in cmd]
+
+    if prlimit:
+        if os.name == 'nt':
+            LOG.log(loglevel,
+                    _('Process resource limits are ignored as '
+                      'this feature is not supported on Windows.'))
+        else:
+            args = [sys.executable, '-m', 'oslo_concurrency.prlimit']
+            args.extend(prlimit.prlimit_args())
+            args.append('--')
+            args.extend(cmd)
+            cmd = args
+
     sanitized_cmd = strutils.mask_password(' '.join(cmd))
 
+    watch = timeutils.StopWatch()
     while attempts > 0:
         attempts -= 1
+        watch.restart()
+
         try:
-            start_time = time.time()
             LOG.log(loglevel, _('Running cmd (subprocess): %s'), sanitized_cmd)
             _PIPE = subprocess.PIPE  # pylint: disable=E1101
 
@@ -249,22 +375,32 @@ def execute(*cmd, **kwargs):
 
                 obj.stdin.close()  # pylint: disable=E1101
                 _returncode = obj.returncode  # pylint: disable=E1101
-                end_time = time.time() - start_time
                 LOG.log(loglevel, 'CMD "%s" returned: %s in %0.3fs',
-                        sanitized_cmd, _returncode, end_time)
+                        sanitized_cmd, _returncode, watch.elapsed())
             finally:
                 if on_completion:
                     on_completion(obj)
 
             if not ignore_exit_code and _returncode not in check_exit_code:
                 (stdout, stderr) = result
+                if six.PY3:
+                    stdout = os.fsdecode(stdout)
+                    stderr = os.fsdecode(stderr)
                 sanitized_stdout = strutils.mask_password(stdout)
                 sanitized_stderr = strutils.mask_password(stderr)
                 raise ProcessExecutionError(exit_code=_returncode,
                                             stdout=sanitized_stdout,
                                             stderr=sanitized_stderr,
                                             cmd=sanitized_cmd)
-            return result
+            if six.PY3 and not binary and result is not None:
+                (stdout, stderr) = result
+                # Decode from the locale using using the surrogateescape error
+                # handler (decoding cannot fail)
+                stdout = os.fsdecode(stdout)
+                stderr = os.fsdecode(stderr)
+                return (stdout, stderr)
+            else:
+                return result
 
         except (ProcessExecutionError, OSError) as err:
             # if we want to always log the errors or if this is
@@ -334,7 +470,13 @@ def trycmd(*args, **kwargs):
 
 
 def ssh_execute(ssh, cmd, process_input=None,
-                addl_env=None, check_exit_code=True):
+                addl_env=None, check_exit_code=True,
+                binary=False, timeout=None):
+    """Run a command through SSH.
+
+    .. versionchanged:: 1.9
+       Added *binary* optional parameter.
+    """
     sanitized_cmd = strutils.mask_password(cmd)
     LOG.debug('Running cmd (SSH): %s', sanitized_cmd)
     if addl_env:
@@ -344,30 +486,53 @@ def ssh_execute(ssh, cmd, process_input=None,
         # This is (probably) fixable if we need it...
         raise InvalidArgumentError(_('process_input not supported over SSH'))
 
-    stdin_stream, stdout_stream, stderr_stream = ssh.exec_command(cmd)
+    stdin_stream, stdout_stream, stderr_stream = ssh.exec_command(
+        cmd, timeout=timeout)
     channel = stdout_stream.channel
 
     # NOTE(justinsb): This seems suspicious...
     # ...other SSH clients have buffering issues with this approach
     stdout = stdout_stream.read()
-    sanitized_stdout = strutils.mask_password(stdout)
     stderr = stderr_stream.read()
-    sanitized_stderr = strutils.mask_password(stderr)
 
     stdin_stream.close()
 
     exit_status = channel.recv_exit_status()
 
+    if six.PY3:
+        # Decode from the locale using using the surrogateescape error handler
+        # (decoding cannot fail). Decode even if binary is True because
+        # mask_password() requires Unicode on Python 3
+        stdout = os.fsdecode(stdout)
+        stderr = os.fsdecode(stderr)
+    stdout = strutils.mask_password(stdout)
+    stderr = strutils.mask_password(stderr)
+
     # exit_status == -1 if no exit code was returned
     if exit_status != -1:
         LOG.debug('Result was %s' % exit_status)
         if check_exit_code and exit_status != 0:
             raise ProcessExecutionError(exit_code=exit_status,
-                                        stdout=sanitized_stdout,
-                                        stderr=sanitized_stderr,
+                                        stdout=stdout,
+                                        stderr=stderr,
                                         cmd=sanitized_cmd)
 
-    return (sanitized_stdout, sanitized_stderr)
+    if binary:
+        if six.PY2:
+            # On Python 2, stdout is a bytes string if mask_password() failed
+            # to decode it, or an Unicode string otherwise. Encode to the
+            # default encoding (ASCII) because mask_password() decodes from
+            # the same encoding.
+            if isinstance(stdout, unicode):
+                stdout = stdout.encode()
+            if isinstance(stderr, unicode):
+                stderr = stderr.encode()
+        else:
+            # fsencode() is the reverse operation of fsdecode()
+            stdout = os.fsencode(stdout)
+            stderr = os.fsencode(stderr)
+
+    return (stdout, stderr)
 
 
 def get_worker_count():
